;****************************************************************
;		  PCE VBL and HBL IRQ Handlers			*
;****************************************************************
; IRQ_VDC:							*
;	IRQ Handler for the VDC.				*
;****************************************************************
.section "IRQ_VDC" force
IRQ_VDC:
	PHA
	LDA VDC_REG		; Ack IRQ
	BIT #VDC_VBL		; Was it VBL?
	bne _gotVBL
	BIT #VDC_RR		; What about HBL?
	bne _gotHBL
	PLA
	rti			; Well, how did we get here?!

_gotVBL
	PHX			; -- VBL --
	PHY			; HBL will be dropped if occuring simultaneously
	jmp IRQ_VBL		; with VBL. (If this is even possible)...
_gotHBL
	PLA			; -- HBL --
	rti			; Nothing for now
.ends


;****************************************************************
; IRQ_VBL:							*
;	End of Frame Updates. Performs any transfer requests	*
; such as tile, scroll, or palette updates as well as reading 	*
; in the new joypad values. Should only be JUMPED to after	*
; pushing A, X, and Y in the VDC IRQ Handler.			*
;****************************************************************
.section "IRQ_VBL" force
IRQ_VBL:
	LDA VBL_Busy		; Are we already in the VBL routine?
	beq _checkFrameDone	; (double entry could happen with super-long
	PLY			; data transfer queues).
	PLX
	PLA
	rti

_checkFrameDone
	LDA #1
	STA VBL_Busy
	jsr HuSound_Writeback	; Immediately handle the audio writebacks.

	LDA FrameDone		; If frame processing hasn't finished, don't
	beq _updateRegs		; copy the VDC mirror variables to the backups,
	LDX #9			; we'll use the values from the last frame.
_copyNewRegStates
	LDA VDCM_CR,X
	STA VDCB_CR,X
	DEX
	bpl _copyNewRegStates

	LDA VCE_BGColor + LO	; ...and the VCE stuff too.
	STA VCE_BGColor_Safe + LO
	LDA VCE_BGColor + HI
	STA VCE_BGColor_Safe + HI


_updateRegs			; Copy our backup VDC registers to the real ones...
	VDC_Copy16 VDC_REG_CR, VDCB_CR
	VDC_Copy16 VDC_REG_RCR, VDCB_RCR
	VDC_Copy16 VDC_REG_BXR, VDCB_BXR
	VDC_Copy16 VDC_REG_BYR, VDCB_BYR
	VDC_Copy16 VDC_REG_MWR, VDCB_MWR

	STZ VCE_CRAM_ADDRLO	; ...don't forget to set the VCE's starting
	STZ VCE_CRAM_ADDRHI	; background color.
	LDA VCE_BGColor_Safe + LO
	STA VCE_CRAM_DATALO
	LDA VCE_BGColor_Safe + HI
	STA VCE_CRAM_DATAHI

_copySAT
	ST0 #VDC_REG_SATB	; DMA our most recently populated SAT to the
	ST1 #0			; VDC's internal copy. If frame processing
	LDY VDC_CurSAT		; hasn't finished, we'll send the last used SAT.
	LDA _sysSATs_Hi,Y
	STA VDC_DATAHI

_flushQueues			; Interrupts are now back on, so we won't have
	cli			; too much jitter on audio sample playback.

	TMA #MPR4		; Save the old object bank and MCGenjin settings
	PHA			; before starting the VCE VRAM transfers...
	TMA #MPR5
	PHA
	LDA MCG_LastMap
	PHA
	jsr VDC_FlushVREQs
	PLA
	STA MCG_LastMap
	STA MCGENJIN_BANK
	PLA
	TAM #MPR5
	PLA
	TAM #MPR4		; ..and restore them after we're done.

	jsr VCE_FlushRing	; VCE Palette transfers can just do their thing.

	STZ VBL_Busy		; Done with actual VBL stuff, so let's
	jsr HuSound_Main	; handle the next audio driver tick.

	STZ FrameDone		; Let the game routine know that it can
	PLY			; proceed to the next frame's processing,
	PLX			; and we're outta here.
	PLA
	rti

; Base addresses for the two SATs kept in VRAM
; (high byte only, assumed to be aligned).
_sysSATs_Hi	.db >VMEM_SCBA,>VMEM_SCBB
.ends
