;****************************************************************
;			  PCE VCE Helpers			*
;****************************************************************
; -VCE_LoadPal:
;	Load a palette directly.
; -VCE_LoadPalAdj:
;	Load a palette with additive RGBI adjustments.
;
; -VCE_FlushRing:
;	...colors go out

.section "VCE_RefTables" force
VCE_SlotDestLo:
	.db $00,$20,$40,$60,$80,$A0,$C0,$E0,$00,$20,$40,$60,$80,$A0,$C0,$E0
	.db $00,$20,$40,$60,$80,$A0,$C0,$E0,$00,$20,$40,$60,$80,$A0,$C0,$E0
VCE_SlotDestHi:
	.db $00,$00,$00,$00,$00,$00,$00,$00,$01,$01,$01,$01,$01,$01,$01,$01
	.db $02,$02,$02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$03,$03,$03
.ends

;****************************************************************
; VCE_LoadPal:							*
;	Requests a palette transfer to the VCE and copies its	*
; data to our color cache.					*
;								*
; PAL_Src - Source Address (Lo, Hi, Bank, MCGenjin Map)		*
; PAL_Dest - Destination Palette Slot (0-31)			*
;****************************************************************
.section "VCE_LoadPal" force
VCE_LoadPal:
	PHY

	LDY VCE_TxRing_I
	TYA
	INA
	AND #$1F
_waitRingReady
	CMP VCE_TxRing_O
	beq _waitRingReady

	LDA PAL_Src + BANK
	STA MCG_Bank
	LDA PAL_Src + MCMAP
	STA MCG_Map
	MCG_Sel			; ~! MCGENJIN OBJ SELECT !~

	LDA PAL_Dest
	STA VCE_TxRing_Dest,Y

	LDA #<VCE_TxRing_ColCache
	clc
	ADC VCE_SlotDestLo,Y
	STA PAL_Dest + LO
	LDA #>VCE_TxRing_ColCache
	ADC VCE_SlotDestHi,Y
	STA PAL_Dest + HI

	LDY #31
_cachePalette
	LDA (PAL_Src),Y
	STA (PAL_Dest),Y
	DEY
	bpl _cachePalette

	LDA VCE_TxRing_I
	INA
	AND #$1F
	STA VCE_TxRing_I

	MCG_Res			; =! MCGENJIN OBJ RESTORE !=
	PLY
	rts
.ends
;****************************************************************
; VCE_LoadPalAdj:						*
;	Requests a palette transfer to the VCE and copies its	*
; data to the color cache after performing an RGBI adjustment.	*
;								*
; PAL_Src - Source Address (Lo, Hi, Bank, MCGenjin Map)		*
; PAL_Dest - Destination Palette Slot (0-31)			*
; PAL_AdjRed,Green,Blue,Int - RGBI Adjustment			*
;****************************************************************
.section "VCE_LoadPalAdj" force
VCE_LoadPalAdj:
	PHY

	LDY VCE_TxRing_I
	TYA
	INA
	AND #$1F
_waitRingReady
	CMP VCE_TxRing_O
	beq _waitRingReady

	LDA PAL_Src + BANK
	STA MCG_Bank
	LDA PAL_Src + MCMAP
	STA MCG_Map
	MCG_Sel			; ~! MCGENJIN OBJ SELECT !~

	LDA PAL_Dest
	STA VCE_TxRing_Dest,Y

	LDA #<VCE_TxRing_ColCache
	clc
	ADC VCE_SlotDestLo,Y
	STA PAL_Dest + LO
	LDA #>VCE_TxRing_ColCache
	ADC VCE_SlotDestHi,Y
	STA PAL_Dest + HI

	LDY #31
_copyPalAdj
	LDA (PAL_Src),Y		; Adjust Blue
	AND #%00000111		; -------- -----BBB
	clc
	ADC PAL_AdjBlue
	clc
	ADC PAL_AdjInt
	bpl _checkBlueUpper
	CLA			; 9-Bit RGB, so each color must be clamped
	bra _writeBlue		; between 000 and 111.
_checkBlueUpper
	CMP #%00001000
	bcc _writeBlue
	LDA #%00000111
_writeBlue
	STA (PAL_Dest),Y

	LDA (PAL_Src),Y		; Adjust Red
	LSR			; -------- --RRR---
	LSR
	LSR
	AND #%00000111
	clc
	ADC PAL_AdjRed
	clc
	ADC PAL_AdjInt
	bpl _checkRedUpper
	CLA
	bra _writeRed
_checkRedUpper
	CMP #%00001000
	bcc _writeRed
	LDA #%00000111
_writeRed
	ASL
	ASL
	ASL
	ORA (PAL_Dest),Y
	STA (PAL_Dest),Y

	DEY			; Adjust Green
	LDA (PAL_Src),Y		; -------G GG------
	ROR
	INY
	LDA (PAL_Src),Y
	ROL
	ROL
	ROL
	AND #%00000111
	clc
	ADC PAL_AdjGreen
	clc
	ADC PAL_AdjInt
	bpl _checkGreenUpper
	CLA
	bra _writeGreen
_checkGreenUpper
	CMP #%00001000
	bcc _writeGreen
	LDA #%00000111
_writeGreen
	ROR
	ROR
	ROR
	AND #%11000000
	ORA (PAL_Dest),Y
	STA (PAL_Dest),Y
	CLA
	ROL
	DEY
	STA (PAL_Dest),Y

	DEY
	bpl _copyPalAdj		; Done?

	LDA VCE_TxRing_I
	INA
	AND #$1F
	STA VCE_TxRing_I

	MCG_Res			; =! MCGENJIN OBJ RESTORE !=
	PLY
	rts
.ends




;****************************************************************
; VCE_FlushRing:						*
;	Empties the current palette transfer ringbuffer. This	*
; should be called only in VBL, but is a safe routine which may	*
; be run whether or not frame processing has completed.		*
;****************************************************************
.section "VCE_FlushRing" force
VCE_FlushRing:
	LDX VCE_TxRing_O
	CPX VCE_TxRing_I
	bne _doPaletteCopies
	rts

_doPaletteCopies
	LDY VCE_TxRing_Dest,X

	LDA #<VCE_TxRing_ColCache
	clc
	ADC VCE_SlotDestLo,Y
	STA VCE_CopySrc + LO
	LDA #>VCE_TxRing_ColCache
	ADC VCE_SlotDestHi,Y
	STA VCE_CopySrc + HI

	TYA
	jsr VCE_CopyPal

	TXA
	INA
	AND #$1F
	TAX
	STX VCE_TxRing_O
	CPX VCE_TxRing_I
	bne _doPaletteCopies
	rts
.ends
;****************************************************************
; VCE_CopyPal:							*
;	Copies the palette whose address is in VCE_CopySrc to	*
; the VCE palette slot in A (0-31). A is trashed.		*
;****************************************************************
.section "VCE_CopyPal" force
VCE_CopyPal:
	PHY
	TAY			; Convert palette index to VCE
	LDA VCE_SlotDestLo,Y	; color address.
	STA VCE_CRAM_ADDRLO	;         %xxxSSSSS
	LDA VCE_SlotDestHi,Y	; %xxxxxxxAAAAA0000
	STA VCE_CRAM_ADDRHI

	LDY #1
_copyToVCE
	LDA (VCE_CopySrc),Y
	STA VCE_CRAM_DATALO
	DEY
	LDA (VCE_CopySrc),Y
	STA VCE_CRAM_DATAHI
	TYA
	ADC #3
	TAY
	CPY #32
	bmi _copyToVCE
	PLY
	rts
.ends




;****************************************************************
; VCE_ClearCRAM:						*
;	Zero palette memory.					*
;****************************************************************
.section "VCE_ClearCRAM" force
VCE_ClearCRAM:
	STZ VCE_CRAM_ADDRLO
	STZ VCE_CRAM_ADDRHI

	LDX #31
_clearslot
	LDY #15
_clearkey
	STZ VCE_CRAM_DATALO
	STZ VCE_CRAM_DATAHI
	DEY
	bpl _clearkey
	DEX
	bpl _clearslot
	rts
.ends
