;****************************************************************
;		  HuSound Pertinent Variables			*
;****************************************************************
.enum HUSOUND_MEM_BASE

; -- Driver State --
;****************************************************************
HuSound_RelAddr		ds 4			; Lo, Hi, Bank, McMap


; -- Channel Properties --
;****************************************************************
; Attributes
;------------
HuSound_WaveMode	ds HUSOUND_CHANNELS	; ($00 = Wave, $FF = Noise)
HuSound_isInstrument	ds HUSOUND_CHANNELS	; ($00 = SFX, $FF = Instrument)

; Writeback Settings
;--------------------
HuSound_noWriteback	ds HUSOUND_CHANNELS	; (Nonzero = Disable writing to channel regs)
HuSound_writeWave	ds HUSOUND_CHANNELS	; (Nonzero = Update)
HuSound_writeFreq	ds HUSOUND_CHANNELS

; Waveform Mirrors
;------------------
HuSound_WaveMem		ds (32 * HUSOUND_CHANNELS)

; Frequency
;-----------
HuSound_FreqBase_Lo	ds HUSOUND_CHANNELS	; The base frequency for instruments
HuSound_FreqBase_Hi	ds HUSOUND_CHANNELS	; directly from the music script,
HuSound_FreqBase_De	ds HUSOUND_CHANNELS	; zeroed during sound effect playback.

HuSound_FreqPitch_Lo	ds HUSOUND_CHANNELS	; Adjustment from the base frequency
HuSound_FreqPitch_Hi	ds HUSOUND_CHANNELS	; for use in music scripts as pitch
HuSound_FreqPitch_De	ds HUSOUND_CHANNELS	; bends or adjustments. Zeroed for
HuSound_FreqPitchAdj_Lo	ds HUSOUND_CHANNELS	; sound effect playback.
HuSound_FreqPitchAdj_Hi	ds HUSOUND_CHANNELS
HuSound_FreqPitchAdj_De	ds HUSOUND_CHANNELS	; Pitch += PitchAdj

HuSound_FreqOff_Lo	ds HUSOUND_CHANNELS	; Frequency adjustment for use in
HuSound_FreqOff_Hi	ds HUSOUND_CHANNELS	; instrument and sound effects.
HuSound_FreqOff_De	ds HUSOUND_CHANNELS
HuSound_FreqOffAdj_Lo	ds HUSOUND_CHANNELS
HuSound_FreqOffAdj_Hi	ds HUSOUND_CHANNELS
HuSound_FreqOffAdj_De	ds HUSOUND_CHANNELS	; Offset += OffsetAdj

HuSound_FreqFinal_Lo	ds HUSOUND_CHANNELS
HuSound_FreqFinal_Hi	ds HUSOUND_CHANNELS	; Final Frequency = Base + Pitch + Offset
HuSound_LastFreq_Lo	ds HUSOUND_CHANNELS
HuSound_LastFreq_Hi	ds HUSOUND_CHANNELS

; Volume
;--------
HuSound_Vol_Last	ds HUSOUND_CHANNELS
HuSound_Vol_Lo		ds HUSOUND_CHANNELS
HuSound_Vol_De		ds HUSOUND_CHANNELS
HuSound_VolAdj_Lo	ds HUSOUND_CHANNELS
HuSound_VolAdj_De	ds HUSOUND_CHANNELS

HuSound_Panning		ds HUSOUND_CHANNELS


; -- Instrument / SFX Script Decoding --
;****************************************************************
; Sound Effects Directory Table
;-------------------------------
HuSFX_PackBase		ds 4			; Lo, Hi, Bank, McMap
HuSFX_DirBase		ds 4			; Lo, Hi, Bank, McMap
HuSFX_DirOffset		ds 4			; Lo, Hi, Bank, McMap
HuSFX_NumSFX		ds 1

; Playback State
;----------------
HuSFX_Delay		ds HUSOUND_CHANNELS

HuSFX_CurStream		ds (4 * HUSOUND_CHANNELS)
HuSFX_NoteOff		ds (4 * HUSOUND_CHANNELS)
HuSFX_DecoStack		ds (4 * HUSOUND_STACKLEN * HUSOUND_CHANNELS)
HuSFX_StackLevel	ds HUSOUND_CHANNELS
HuSFX_LoopCount		ds (HUSOUND_STACKLEN * HUSOUND_CHANNELS)

; Play Requests and Dispatcher
;------------------------------
HuSFX_ReqSorted		ds HUSOUND_REQLEN
HuSFX_ReqAlloc		ds HUSOUND_CHANNELS

HuSFX_ReqNum		ds HUSOUND_REQLEN
HuSFX_ReqPri		ds HUSOUND_REQLEN
HuSFX_ReqPan		ds HUSOUND_REQLEN
HuSFX_ReqCha		ds HUSOUND_REQLEN

HuSFX_DispDesc_Lo	ds HUSOUND_REQLEN
HuSFX_DispDesc_Hi	ds HUSOUND_REQLEN
HuSFX_DispDesc_Bank	ds HUSOUND_REQLEN
HuSFX_DispDesc_Map	ds HUSOUND_REQLEN


; -- Music Script Decoding --
;****************************************************************
; Music Directory Table
;-----------------------
HuMusic_PackBase	ds 4			; Lo, Hi, Bank, McMap
HuMusic_DirBase		ds 4			; Lo, Hi, Bank, McMap
HuMusic_NumTracks	ds 1

HuMusic_SongBase	ds 4			; Lo, Hi, Bank, McMap
HuMusic_InstBase	ds 4			; Lo, Hi, Bank, McMap
HuMusic_InstOffset	ds 4			; Lo, Hi, Bank, McMap
HuMusic_NumInst		ds 1

; Playback State
;----------------
HuMusic_ReqPlay		ds 1
HuMusic_ReqTrack	ds 1

HuMusic_isPlaying	ds 1
HuMusic_ActiveAcc	ds 1
HuMusic_Priority	ds HUSOUND_CHANNELS
HuMusic_Panning		ds HUSOUND_CHANNELS
HuMusic_Delay		ds HUSOUND_CHANNELS

HuMusic_CurStream	ds (4 * HUSOUND_CHANNELS)
HuMusic_DecoStack	ds (4 * HUSOUND_STACKLEN * HUSOUND_CHANNELS)
HuMusic_StackLevel	ds HUSOUND_CHANNELS
HuMusic_LoopCount	ds (HUSOUND_STACKLEN * HUSOUND_CHANNELS)


; -- PCM Playback --
;****************************************************************
; Sample Directory Table
;------------------------
HuSample_DirBase	ds 4			; Lo, Hi, Bank, McMap
HuSample_NumSamples	ds 1

; Playback State
;----------------
HuSample_Priority	ds HUSOUND_CHANNELS

HuSample_Src_Lo		ds HUSOUND_CHANNELS
HuSample_Src_Hi		ds HUSOUND_CHANNELS
HuSample_Src_Bank	ds HUSOUND_CHANNELS
HuSample_Src_McMap	ds HUSOUND_CHANNELS	; Current source addresses

HuSample_Rem_Lo		ds HUSOUND_CHANNELS
HuSample_Rem_Md		ds HUSOUND_CHANNELS
HuSample_Rem_Hi		ds HUSOUND_CHANNELS	; Sample data remaining


; Segment End
;-------------
HUSOUND_MEM_END		ds 1
.ende
.printt "HuSound : $"
.printv HEX HUSOUND_MEM_BASE
.printt " to $"
.printv HEX HUSOUND_MEM_END - 1
.printt ", "
.printv DEC HUSOUND_MEM_END - HUSOUND_MEM_BASE
.printt " bytes\n"
